package dev.fyloz.colorrecipesexplorer.service

import dev.fyloz.colorrecipesexplorer.config.annotations.ServiceComponent
import dev.fyloz.colorrecipesexplorer.dtos.CompanyDto
import dev.fyloz.colorrecipesexplorer.model.Company
import dev.fyloz.colorrecipesexplorer.repository.CompanyRepository

interface CompanyService : Service<CompanyDto, Company, CompanyRepository> {
    /** Checks if a company with the given [name] exists. */
    fun existsByName(name: String, id: Long?): Boolean

    /** Checks if a recipe depends on the company with the given [id]. */
    fun isUsedByRecipe(id: Long): Boolean
}

@ServiceComponent
class DefaultCompanyService(repository: CompanyRepository) :
    BaseService<CompanyDto, Company, CompanyRepository>(repository), CompanyService {
    override fun existsByName(name: String, id: Long?) = repository.existsByNameAndIdNot(name, id ?: 0)
    override fun isUsedByRecipe(id: Long) = repository.isUsedByRecipe(id)

    override fun toDto(entity: Company) =
        CompanyDto(entity.id!!, entity.name)

    override fun toEntity(dto: CompanyDto) =
        Company(dto.id, dto.name)
}